@@ -19,7 +19,7 @@ BEGIN {
 use Apache2::RequestRec ();
 use Apache2::RequestIO ();
 use Apache2::RequestUtil ();
-use Apache2::Const -compile => qw(FORBIDDEN OK REDIRECT);
+use Apache2::Const -compile => qw(:common);
 
 use Apache2::ServerRec ();
 use Apache2::ServerUtil ();
